# =XMPP4R - XMPP Library for Ruby
# License:: Ruby's license (see the LICENSE file) or GNU GPL, at your option.
# Website::http://xmpp4r.github.io

require 'xmpp4r/dataforms'
require 'xmpp4r/muc/x/muc'
require 'xmpp4r/muc/x/mucuserinvite'
require 'xmpp4r/muc/x/mucuseritem'
require 'xmpp4r/muc/iq/mucowner'
require 'xmpp4r/muc/iq/mucadmin'
require 'xmpp4r/muc/iq/mucadminitem'
require 'xmpp4r/muc/helper/mucbrowser'
require 'xmpp4r/muc/helper/mucclient'
require 'xmpp4r/muc/helper/simplemucclient'
