# =XMPP4R - XMPP Library for Ruby
# License:: Ruby's license (see the LICENSE file) or GNU GPL, at your option.
# Website::http://xmpp4r.github.io

require 'xmpp4r/vcard/helper/vcard.rb'
require 'xmpp4r/vcard/iq/vcard.rb'
