# =XMPP4R - XMPP Library for Ruby
#
# This file's copyright (c) 2009 by Pablo Lorenzzoni <pablo@propus.com.br>
# License:: Ruby's license (see the LICENSE file) or GNU GPL, at your option.
# Website::http://xmpp4r.github.io

require 'xmpp4r/observable/thread_store.rb'
require 'xmpp4r/observable/observable_thing.rb'
require 'xmpp4r/observable/helper.rb'
