# =XMPP4R - XMPP Library for Ruby
# License:: Ruby's license (see the LICENSE file) or GNU GPL, at your option.
# Website::http://xmpp4r.github.io

require 'xmpp4r/roster/iq/roster.rb'
require 'xmpp4r/roster/helper/roster.rb'
require 'xmpp4r/roster/x/roster.rb'
